@@ -1,7 +1,9 @@
+import os
 import logging
 import math
 import yaml
 import random
+import shutil
 import warnings
 from tqdm import tqdm
 
@@ -44,7 +46,9 @@ class TileConfig:
                  valid_ratio: float = 0.1,
                  test_ratio: float = 0.1,
                  margins: Union[float, Tuple[float, float, float, float]] = 0.0,
-                 include_negative_samples: bool = True):
+                 include_negative_samples: bool = True,
+                 copy_source_data: bool = False,
+                 compression: int = 90):
         """
         Args:
             slice_wh: Size of each slice (width, height)
@@ -59,6 +63,8 @@ class TileConfig:
             test_ratio: Ratio of test set
             margins: Margins to exclude from tiling (left, top, right, bottom)
             include_negative_samples: Include tiles without annotations
+            copy_source_data: Copy original source images to target directory
+            compression: Compression percentage for different output formats (0-100)
         """
         self.slice_wh = slice_wh if isinstance(slice_wh, tuple) else (slice_wh, slice_wh)
         self.overlap_wh = overlap_wh
@@ -71,11 +77,14 @@ class TileConfig:
         self.valid_ratio = valid_ratio
         self.test_ratio = test_ratio
         self.include_negative_samples = include_negative_samples
+        self.copy_source_data = copy_source_data
+        self.compression = compression
         
         # Validate annoation type
-        if self.annotation_type not in ["object_detection", "instance_segmentation", "image_classification"]:
+        valid_types = ["object_detection", "instance_segmentation", "image_classification", "semantic_segmentation"]
+        if self.annotation_type not in valid_types:
             raise ValueError(f"ERROR: Invalid annotation type: {self.annotation_type}\n\
-                              Must be one of 'object_detection', 'instance_segmentation', 'image_classification'")
+                              Must be one of {valid_types}")
 
         # Handle margins
         if isinstance(margins, (int, float)):
@@ -149,15 +158,19 @@ class YoloTiler:
             show_processing_status: Whether to show processing status
             progress_callback: Optional callback function to report progress
         """
-        self.source = Path(source)
-        self.target = Path(target)
-        self.config = config
-        self.num_viz_samples = num_viz_samples
-        
         # Add show_process_status parameter and initialize progress bars dict
         self.show_process_status = show_processing_status
         self._progress_bars = {}
         
+        try:
+            self.source = Path(source)
+            self.target = Path(target)
+        except:
+            raise ValueError("Source and target must be valid paths")
+        
+        self.config = config
+        self.num_viz_samples = num_viz_samples
+        
         # Set up the progress callback based on parameters
         if progress_callback is not None:
             self.progress_callback = progress_callback
@@ -271,7 +284,7 @@ class YoloTiler:
                     if not class_folders:
                         raise ValueError(f"No class folders found in {folder / subfolder}")
             else:
-                # Check for images and labels folders for object detection and instance segmentation
+                # Check for images and labels folders for detection and segmentation tasks
                 if not (folder / subfolder / 'images').exists():
                     raise ValueError(f"Required folder {folder / subfolder / 'images'} does not exist")
                 if not (folder / subfolder / 'labels').exists():
@@ -478,6 +491,22 @@ class YoloTiler:
             df = pd.DataFrame(labels, columns=['class', 'x1', 'y1', 'w', 'h'])
             df.to_csv(path, sep=' ', index=False, header=False, float_format='%.6f')
 
+    def _save_mask(self, mask: np.ndarray, path: Path) -> None:
+        """
+        Save mask to PNG file for semantic segmentation.
+
+        Args:
+            mask: Numpy array of mask data
+            path: Path to save mask
+        """
+        # Ensure mask is uint8
+        mask = mask.astype(np.uint8)
+        
+        # Save as PNG
+        from PIL import Image
+        img = Image.fromarray(mask, mode='L')  # 'L' for grayscale
+        img.save(path)
+
     def tile_image(self, 
                    image_path: Path, 
                    label_path: Union[Path, str],  # Path to labels.txt, or class name
@@ -518,8 +547,23 @@ class YoloTiler:
             # Calculate total tiles for progress tracking
             total_tiles = self._count_total_tiles((effective_width, effective_height))
             
-            # Read all lines from label file if object detection or instance segmentation
-            if self.annotation_type != "image_classification":
+            # Read labels based on annotation type
+            if self.annotation_type == "image_classification":
+                # Image classification (unnecessary for tiling)
+                lines = []
+                boxes = []
+                mask_data = None
+            elif self.annotation_type == "semantic_segmentation":
+                # Read PNG mask for semantic segmentation
+                try:
+                    with rasterio.open(label_path) as mask_src:
+                        mask_data = mask_src.read(1)  # Read single channel mask
+                    lines = []
+                    boxes = []
+                except Exception as e:
+                    raise ValueError(f"Failed to read mask file {label_path}: {e}")
+            else:
+                # Object detection and instance segmentation - read text file
                 try:
                     f = open(label_path)
                     lines = f.readlines()
@@ -527,14 +571,10 @@ class YoloTiler:
                     
                     # Boxes or polygons
                     boxes = []
+                    mask_data = None
                     
                 except Exception as e:
                     raise ValueError(f"Failed to read label file {label_path}: {e}")
-                
-            else:  # Image classification (unnecessary for tiling)
-                lines = []
-                boxes = []
-                tile_labels = []
             
             # Process each line
             for line in lines:
@@ -637,35 +677,40 @@ class YoloTiler:
 
                 tile_labels = []
 
-                # Process annotations for this tile (boxes or polygons)
-                for box_class, box_polygon in boxes:
-                    # Deal with intersections if they occur
-                    # This is necessary for cases where a box / polygon
-                    # is split across tiles
-                    if tile_polygon.intersects(box_polygon):
-                        intersection = tile_polygon.intersection(box_polygon)
+                if self.annotation_type == "semantic_segmentation":
+                    # Crop the mask to the tile area
+                    tile_mask = mask_data[abs_y1:abs_y2, abs_x1:abs_x2]
+                    tile_labels = tile_mask
+                else:
+                    # Process annotations for this tile (boxes or polygons)
+                    for box_class, box_polygon in boxes:
+                        # Deal with intersections if they occur
+                        # This is necessary for cases where a box / polygon
+                        # is split across tiles
+                        if tile_polygon.intersects(box_polygon):
+                            intersection = tile_polygon.intersection(box_polygon)
 
-                        if self.config.annotation_type == "object_detection":
-                            # Handle object detection
-                            bbox = intersection.envelope
-                            center = bbox.centroid
-                            bbox_coords = bbox.exterior.coords.xy
+                            if self.config.annotation_type == "object_detection":
+                                # Handle object detection
+                                bbox = intersection.envelope
+                                center = bbox.centroid
+                                bbox_coords = bbox.exterior.coords.xy
 
-                            # Normalize relative to tile dimensions
-                            tile_width = abs_x2 - abs_x1
-                            tile_height = abs_y2 - abs_y1
+                                # Normalize relative to tile dimensions
+                                tile_width = abs_x2 - abs_x1
+                                tile_height = abs_y2 - abs_y1
 
-                            new_width = (max(bbox_coords[0]) - min(bbox_coords[0])) / tile_width
-                            new_height = (max(bbox_coords[1]) - min(bbox_coords[1])) / tile_height
-                            new_x = (center.x - abs_x1) / tile_width
-                            new_y = (center.y - abs_y1) / tile_height
+                                new_width = (max(bbox_coords[0]) - min(bbox_coords[0])) / tile_width
+                                new_height = (max(bbox_coords[1]) - min(bbox_coords[1])) / tile_height
+                                new_x = (center.x - abs_x1) / tile_width
+                                new_y = (center.y - abs_y1) / tile_height
 
-                            tile_labels.append([box_class, new_x, new_y, new_width, new_height])
-                        else:
-                            # Handle instance segmentation
-                            coord_lists = self._process_intersection(intersection)
-                            normalized = self._normalize_coordinates(coord_lists, (abs_x1, abs_y1, abs_x2, abs_y2))
-                            tile_labels.append([box_class, normalized])
+                                tile_labels.append([box_class, new_x, new_y, new_width, new_height])
+                            else:
+                                # Handle instance segmentation
+                                coord_lists = self._process_intersection(intersection)
+                                normalized = self._normalize_coordinates(coord_lists, (abs_x1, abs_y1, abs_x2, abs_y2))
+                                tile_labels.append([box_class, normalized])
 
                 # Save tile image and labels if include_negative_samples is True or there are labels
                 if self.config.include_negative_samples or tile_labels:
@@ -678,7 +723,7 @@ class YoloTiler:
     def _save_tile_image(self, tile_data: np.ndarray, image_path: Path, suffix: str, folder: str) -> None:
         """
         Save a tile image to the appropriate directory.
-
+    
         Args:
             tile_data: Numpy array of tile image
             image_path: Path to original image
@@ -687,28 +732,58 @@ class YoloTiler:
         """
         # Set the save directory based on annotation type
         if self.annotation_type == "image_classification":
-            # For image classification, the directory includes the class category (already validated)
-            # Extract class from original path (assuming structure: source/train/class/image.jpg)
             class_name = image_path.parent.name
             save_dir = self.target / folder / class_name
             image_path_out = save_dir / image_path.name.replace(self.config.input_ext, suffix)
         else:
-            # For object detection/segmentation
             save_dir = self.target / folder / "images"
             image_path_out = save_dir / image_path.name.replace(self.config.input_ext, suffix)
-
+    
         # Make sure the directory exists
         save_dir.mkdir(parents=True, exist_ok=True)
-
-        # Save the image
+    
+        # Select appropriate driver and options based on extension
+        output_ext = self.config.output_ext.lower()
+        
+        if output_ext in ['.jpg', '.jpeg']:
+            driver = 'JPEG'
+            options = {'quality': self.config.compression}
+        elif output_ext == '.png':
+            driver = 'PNG'
+            # PNG compression (0-9); convert from JPEG scale
+            png_compression = min(9, max(0, int(self.config.compression / 10)))
+            options = {'zlevel': png_compression}
+        elif output_ext == '.bmp':
+            driver = 'BMP'
+            options = {}  # BMP doesn't support compression
+        elif output_ext in ['.tif', '.tiff']:
+            driver = 'GTiff'
+            # Map JPEG quality to appropriate TIFF compression
+            if self.config.compression >= 90:
+                # Use lossless LZW for high quality
+                options = {'compress': 'lzw'}
+            elif self.config.compression >= 75:
+                # Use lossless DEFLATE for medium-high quality
+                options = {'compress': 'deflate', 'zlevel': 6}
+            else:
+                # Use JPEG compression for lower quality settings
+                # Scale the JPEG quality to something reasonable for TIFF
+                tiff_jpeg_quality = max(50, self.config.compression)
+                options = {'compress': 'jpeg', 'jpeg_quality': tiff_jpeg_quality}
+        else:
+            # Default to GTiff for unknown formats
+            driver = 'GTiff'
+            options = {'compress': 'lzw'}  # Safe default
+    
         with rasterio.open(
             image_path_out,
             'w',
-            driver='GTiff',
+            driver=driver,
             height=tile_data.shape[1],
             width=tile_data.shape[2],
             count=tile_data.shape[0],
-            dtype=tile_data.dtype
+            dtype=tile_data.dtype,
+            **options
         ) as dst:
             dst.write(tile_data)
 
@@ -723,10 +798,22 @@ class YoloTiler:
             folder: Subfolder name (train, valid, test)
         """
         # Save the labels in the appropriate directory
-        label_path = self.target / folder / "labels" / image_path.name.replace(self.config.input_ext, suffix)
-        label_path = label_path.with_suffix('.txt')
-        
-        if self.annotation_type != "image_classification":
+        if self.annotation_type == "semantic_segmentation":
+            # For semantic segmentation, always use PNG format for masks
+            # Extract coordinates from suffix and create PNG suffix
+            parts = suffix.split('_')
+            if len(parts) >= 5:
+                x1, y1, width, height = parts[-4], parts[-3], parts[-2], parts[-1].split('.')[0]
+                mask_suffix = f'_{x1}_{y1}_{width}_{height}.png'
+            else:
+                # Fallback if parsing fails
+                mask_suffix = suffix.replace(self.config.output_ext, '.png')
+            label_path = self.target / folder / "labels" / image_path.name.replace(self.config.input_ext, mask_suffix)
+            # labels is a numpy array for semantic segmentation
+            self._save_mask(labels, label_path)
+        elif self.annotation_type != "image_classification":
+            label_path = self.target / folder / "labels" / image_path.name.replace(self.config.input_ext, suffix)
+            label_path = label_path.with_suffix('.txt')
             self._save_labels(labels, 
                               label_path, 
                               is_segmentation=self.annotation_type == "instance_segmentation")
@@ -753,7 +840,7 @@ class YoloTiler:
         
         self._save_tile_image(tile_data, original_path, suffix, folder)
         
-        # Only save label files for object detection and instance segmentation
+        # Only save label files for detection and segmentation tasks
         if self.annotation_type != "image_classification":
             self._save_tile_labels(labels, original_path, suffix, folder)
 
@@ -776,7 +863,24 @@ class YoloTiler:
             self.logger.warning("No train data found to split")
             return
 
-        combined = list(zip(train_images, train_labels))
+        # Create a dictionary mapping from image stem to image path
+        image_dict = {img_path.stem: img_path for img_path in train_images}
+        
+        # Create a dictionary mapping from label stem to label path
+        label_dict = {lbl_path.stem: lbl_path for lbl_path in train_labels}
+        
+        # Create properly matched image-label pairs
+        combined = []
+        for stem, img_path in image_dict.items():
+            if stem in label_dict:
+                combined.append((img_path, label_dict[stem]))
+            else:
+                self.logger.warning(f"No matching label found for image: {img_path.name}, skipping")
+        
+        if not combined:
+            self.logger.warning("No matching image-label pairs found to split")
+            return
+                
         random.shuffle(combined)
 
         num_train = int(len(combined) * self.config.train_ratio)
@@ -914,9 +1018,21 @@ class YoloTiler:
             image_paths = list((self.source / subfolder).glob(f'**/*{self.config.input_ext}'))
             label_paths = [image_path.parent.name for image_path in image_paths]
         else:
-            # Object detection and instance segmentation (get the images and labels in subfolders)
+            # Detection and segmentation tasks (get the images and labels in subfolders)
             image_paths = list((self.source / subfolder / 'images').glob(f'*{self.config.input_ext}'))
-            label_paths = list((self.source / subfolder / 'labels').glob('*.txt'))
+            
+            if self.annotation_type == "semantic_segmentation":
+                label_paths = list((self.source / subfolder / 'labels').glob('*.png'))
+            else:
+                label_paths = list((self.source / subfolder / 'labels').glob('*.txt'))
+            
+            # Sort paths to ensure consistent ordering
+            image_paths.sort()
+            label_paths.sort()
+            
+            # For detection/segmentation, create a mapping of stem to label path
+            # This ensures correct matching regardless of directory listing order
+            label_dict = {path.stem: path for path in label_paths}
 
         # Log the number of images, labels found
         self.logger.info(f'Found {len(image_paths)} images in {subfolder} directory')
@@ -933,10 +1049,16 @@ class YoloTiler:
         total_images = len(image_paths)
 
         # Process each image
-        for current_image_idx, (image_path, label_path) in enumerate(zip(image_paths, label_paths)):
+        for current_image_idx, image_path in enumerate(image_paths):
             if self.annotation_type != "image_classification":
-                if image_path.stem != label_path.stem:
-                    raise ValueError(f"Image and label file names do not match: {image_path.name}, {label_path.name}")
+                # Look up the matching label path based on stem instead of position
+                label_path = label_dict.get(image_path.stem)
+                if label_path is None:
+                    self.logger.warning(f"No matching label found for image: {image_path.name}, skipping")
+                    continue
+            else:
+                # For classification, the label is still the parent folder name
+                label_path = image_path.parent.name
             
             self.logger.info(f'Processing {image_path}')
             self.tile_image(image_path, label_path, subfolder, current_image_idx + 1, total_images)
@@ -979,12 +1101,61 @@ class YoloTiler:
                     data['valid'] = str(self.target / 'valid' / 'images')
                 if 'test' in data:
                     data['test'] = str(self.target / 'test' / 'images')
+                if 'path' in data:
+                    data['path'] = str(self.target)
                 
                 # Write updated YAML
                 with open(self.target / 'data.yaml', 'w') as f:
                     yaml.dump(data, f, sort_keys=False)
             else:
                 self.logger.warning('data.yaml not found in source directory')
+                
+    def _copy_source_data(self) -> None:
+        """Copy original source data to the target directory."""        
+        self.logger.info('Copying original source data to target directory...')
+        
+        for subfolder in self.subfolders:
+            if self.annotation_type == "image_classification":
+                # For image classification, copy all class directories
+                source_dir = self.source / subfolder
+                if source_dir.exists():
+                    # Get all class directories
+                    class_dirs = [d for d in source_dir.iterdir() if d.is_dir()]
+                    
+                    for class_dir in class_dirs:
+                        class_name = class_dir.name
+                        target_class_dir = self.target / f"{subfolder.rstrip('/')}" / class_name
+                        target_class_dir.mkdir(parents=True, exist_ok=True)
+                        
+                        # Copy all images for this class
+                        for img_path in class_dir.glob(f"*{self.config.input_ext}"):
+                            shutil.copy2(img_path, target_class_dir / img_path.name)
+            else:
+                # For detection and segmentation tasks
+                source_img_dir = self.source / subfolder / "images"
+                source_lbl_dir = self.source / subfolder / "labels"
+                
+                if source_img_dir.exists() and source_lbl_dir.exists():
+                    target_img_dir = self.target / f"{subfolder.rstrip('/')}" / "images"
+                    target_lbl_dir = self.target / f"{subfolder.rstrip('/')}" / "labels"
+                    
+                    target_img_dir.mkdir(parents=True, exist_ok=True)
+                    target_lbl_dir.mkdir(parents=True, exist_ok=True)
+                    
+                    # Copy all images
+                    for img_path in source_img_dir.glob(f"*{self.config.input_ext}"):
+                        shutil.copy2(img_path, target_img_dir / img_path.name)
+                    
+                    # Copy all labels (TXT for detection/segmentation, PNG for semantic)
+                    if self.annotation_type == "semantic_segmentation":
+                        label_pattern = "*.png"
+                    else:
+                        label_pattern = "*.txt"
+                    
+                    for lbl_path in source_lbl_dir.glob(label_pattern):
+                        shutil.copy2(lbl_path, target_lbl_dir / lbl_path.name)
+        
+        self.logger.info('Source data copied successfully')
 
     def visualize_random_samples(self) -> None:
         """
@@ -1035,8 +1206,11 @@ class YoloTiler:
             if self.annotation_type == "image_classification":
                 source_label = source_image_path.parent.name  # Class name
             else:
-                # For object detection and instance segmentation
-                source_label_path = source_image_path.parent.parent / 'labels' / f"{source_image_path.stem}.txt"
+                # For detection and segmentation tasks
+                if self.annotation_type == "semantic_segmentation":
+                    source_label_path = source_image_path.parent.parent / 'labels' / f"{source_image_path.stem}.png"
+                else:
+                    source_label_path = source_image_path.parent.parent / 'labels' / f"{source_image_path.stem}.txt"
                 if not source_label_path.exists():
                     self.logger.warning(f"Label file not found for source {source_image_path.name}")
                     continue
@@ -1053,8 +1227,11 @@ class YoloTiler:
                 if self.annotation_type == "image_classification":
                     tile_label = tile_path.parent.name  # Class name
                 else:
-                    # For object detection and instance segmentation
-                    tile_label_path = self.target / 'train' / 'labels' / f"{tile_path.stem}.txt"
+                    # For detection and segmentation tasks
+                    if self.annotation_type == "semantic_segmentation":
+                        tile_label_path = self.target / 'train' / 'labels' / f"{tile_path.stem}.png"
+                    else:
+                        tile_label_path = self.target / 'train' / 'labels' / f"{tile_path.stem}.txt"
                     if not tile_label_path.exists():
                         self.logger.warning(f"Label file not found for tile {tile_path.name}")
                         continue
@@ -1124,8 +1301,38 @@ class YoloTiler:
         np.random.seed(42)  # For consistent colors
         colors = np.random.rand(100, 3)  # Support up to 100 classes
         
-        if self.annotation_type != "image_classification":
-            # Read and parse labels
+        if self.annotation_type == "image_classification":
+            class_name = label_path
+            ax.text(width / 2, height / 2, 
+                    class_name, 
+                    fontsize=12, 
+                    color='white', 
+                    backgroundcolor='black',
+                    ha='center')
+        elif self.annotation_type == "semantic_segmentation":
+            # Read and overlay the mask
+            mask = cv2.imread(str(label_path), cv2.IMREAD_GRAYSCALE)
+            if mask is not None:
+                # Ensure dimensions match (H, W)
+                mask = mask.squeeze()
+                # Create a colored overlay
+                colored_mask = np.zeros_like(img)
+                for class_id in np.unique(mask):
+                    if class_id == 0:  # Skip background
+                        continue
+                    color = colors[class_id % len(colors)]
+                    # Convert to RGB values
+                    rgb_color = (int(color[0] * 255), int(color[1] * 255), int(color[2] * 255))
+                    colored_mask[mask == class_id] = rgb_color
+                
+                # Blend with original image
+                alpha = 0.5
+                overlay = cv2.addWeighted(img, 1 - alpha, colored_mask, alpha, 0)
+                ax.imshow(overlay)
+            else:
+                self.logger.warning(f"Could not read mask: {label_path}")
+        else:
+            # Object detection and instance segmentation - read text file
             with open(label_path) as f:
                 for line in f:
                     parts = line.strip().split()
@@ -1158,29 +1365,31 @@ class YoloTiler:
                     else:  # instance segmentation
                         # Parse polygon coordinates
                         coords = []
-                        for i in range(1, len(parts), 2):
-                            x = float(parts[i]) * width
-                            y = float(parts[i + 1]) * height
-                            coords.append([x, y])
+                        try:
+                            for i in range(1, len(parts), 2):
+                                if i + 1 < len(parts):  # Make sure y coordinate exists
+                                    x = float(parts[i]) * width
+                                    y = float(parts[i + 1]) * height
+                                    coords.append([x, y])
 
-                        # Create polygon patch with transparency
-                        polygon = MplPolygon(
-                            coords,
-                            facecolor=color,
-                            edgecolor=color,
-                            linewidth=2,
-                            alpha=0.3  # Add transparency
-                        )
-                        ax.add_patch(polygon)
-                        
-        else:  # Image classification
-            class_name = label_path
-            ax.text(width / 2, height / 2, 
-                    class_name, 
-                    fontsize=12, 
-                    color='white', 
-                    backgroundcolor='black',
-                    ha='center')
+                            # Only create polygon if we have enough valid coordinates
+                            if len(coords) >= 3 and len(set(tuple(p) for p in coords)) >= 3:  # At least 3 unique points
+                                # Create polygon patch with transparency
+                                polygon = MplPolygon(
+                                    coords,
+                                    facecolor=color,
+                                    edgecolor=color,
+                                    linewidth=2,
+                                    alpha=0.3
+                                )
+                                try:
+                                    ax.add_patch(polygon)
+                                except Exception as e:
+                                    self.logger.warning(f"Failed to add polygon: {e}, coords shape")
+                            else:
+                                self.logger.warning(f"Polygon with insufficient coordinates in {image_path.name}")
+                        except Exception as e:
+                            self.logger.warning(f"Error creating polygon in {image_path.name}: {e}")
             
         # Remove axes
         ax.axis('off')
@@ -1211,6 +1420,10 @@ class YoloTiler:
 
             # Copy and update data.yaml with new paths
             self._copy_and_update_data_yaml()
+            
+            # Copy source data if requested
+            if self.config.copy_source_data:
+                self._copy_source_data()
 
             # Generate visualizations if requested
             if self.num_viz_samples > 0:
@@ -1224,6 +1437,7 @@ class YoloTiler:
         
     def __del__(self):
         """Cleanup method to ensure all progress bars are closed"""
-        for pbar in self._progress_bars.values():
-            pbar.close()
-        self._progress_bars.clear()
+        if self._progress_bars:
+            for pbar in self._progress_bars.values():
+                pbar.close()
+            self._progress_bars.clear()
